/*
 * Sonar, open source software quality management tool.
 * Copyright (C) 2009 SonarSource SA
 * mailto:contact AT sonarsource DOT com
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.commons.resources;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.sonar.commons.BaseIdentifiable;
import org.sonar.commons.Metric;
import org.sonar.commons.database.DatabaseSession;
import org.sonar.commons.rules.Rule;
import org.sonar.commons.rules.RulePriority;

import javax.persistence.*;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Entity
@Table(name = "project_measures")
public class Measure extends BaseIdentifiable {

  public static final int TEXT_VALUE_LENGTH = 96;

  @Column(name = "value", updatable = true, nullable = true, precision = 30, scale = 20)
  private Double value = 0.0;

  @Column(name = "text_value", updatable = true, nullable = true, length = TEXT_VALUE_LENGTH)
  private String textValue;

  @Column(name = "tendency", updatable = true, nullable = true)
  private Integer tendency;

  @ManyToOne(fetch = FetchType.EAGER)
  @JoinColumn(name = "metric_id")
  @Cache(usage = CacheConcurrencyStrategy.READ_ONLY)
  private Metric metric;

  @Column(name = "snapshot_id", updatable = true, nullable = true)
  private Integer snapshotId;

  @Column(name = "project_id", updatable = true, nullable = true)
  private Integer projectId;

  @Column(name = "description", updatable = true, nullable = true, length = 4000)
  private String description;

  @Temporal(TemporalType.TIMESTAMP)
  @Column(name = "measure_date", updatable = true, nullable = true)
  private Date measureDate;

  @ManyToOne(fetch = FetchType.LAZY)
  @JoinColumn(name = "rule_id")
  @Cache(usage = CacheConcurrencyStrategy.READ_ONLY)
  private Rule rule;

  @Column(name = "rules_category_id")
  private Integer rulesCategoryId;

  @Column(name = "rule_priority", updatable = false, nullable = true)
  @Enumerated(EnumType.ORDINAL)
  private RulePriority rulePriority;

  @Column(name = "alert_status", updatable = true, nullable = true, length = 5)
  private String alertStatus;

  @Column(name = "alert_text", updatable = true, nullable = true, length = 4000)
  private String alertText;

  @Column(name = "diff_value_1", updatable = true, nullable = true)
  private Double diffValue1;

  @Column(name = "diff_value_2", updatable = true, nullable = true)
  private Double diffValue2;

  @Column(name = "diff_value_3", updatable = true, nullable = true)
  private Double diffValue3;

  @OneToMany(mappedBy = "measure", fetch = FetchType.LAZY, cascade = {CascadeType.PERSIST, CascadeType.MERGE, CascadeType.REMOVE})
  private List<MeasureData> measureData = new ArrayList<MeasureData>();

  public Measure(Metric metric, Double val) {
    if (val.isNaN() || val.isInfinite()) {
      throw new IllegalArgumentException("Measure value is NaN. Metric=" + metric);
    }
    this.metric = metric;
    this.value = val;
  }

  public Measure(Metric metric, Metric.Level level) {
    this.metric = metric;
    if (level != null) {
      this.textValue = level.toString();
    }
  }

  public Measure(Metric metric, String val) {
    this.metric = metric;
    setData(val);
  }

  public Measure() {
  }

  public Double getValue() {
    return value;
  }

  public String getDescription() {
    return description;
  }

  public void setDescription(String description) {
    this.description = description;
  }

  public Measure setValue(Double value) throws IllegalArgumentException {
    if (value != null && (value.isNaN() || value.isInfinite())) {
      throw new IllegalArgumentException();
    }
    this.value = value;
    return this;
  }

  public Metric.Level getLevelValue() {
    if (textValue != null) {
      return Metric.Level.valueOf(textValue);
    }
    return null;
  }

  /**
   * Use getData() instead
   */
  public String getTextValue() {
    return textValue;
  }

  /**
   * Use setData() instead
   */
  public void setTextValue(String textValue) {
    this.textValue = textValue;
  }

  public Integer getTendency() {
    return tendency;
  }

  public boolean isRuleMeasure() {
    return rule != null || rulePriority != null || rulesCategoryId != null;
  }

  public Measure setTendency(Integer tendency) {
    this.tendency = tendency;
    return this;
  }

  public Metric getMetric() {
    return metric;
  }

  public void setMetric(Metric metric) {
    this.metric = metric;
  }

  public Integer getSnapshotId() {
    return snapshotId;
  }

  public Measure setSnapshotId(Integer snapshotId) {
    this.snapshotId = snapshotId;
    return this;
  }

  public Rule getRule() {
    return rule;
  }

  public Measure setRule(Rule rule) {
    this.rule = rule;
    return this;
  }

  public Integer getRulesCategoryId() {
    return rulesCategoryId;
  }

  public Measure setRulesCategoryId(Integer id) {
    this.rulesCategoryId = id;
    return this;
  }

  public RulePriority getRulePriority() {
    return rulePriority;
  }

  public void setRulePriority(RulePriority rulePriority) {
    this.rulePriority = rulePriority;
  }

  public Integer getProjectId() {
    return projectId;
  }

  public void setProjectId(Integer projectId) {
    this.projectId = projectId;
  }

  public Date getMeasureDate() {
    return measureDate;
  }

  public Measure setMeasureDate(Date measureDate) {
    this.measureDate = measureDate;
    return this;
  }

  public Metric.Level getAlertStatus() {
    if (alertStatus == null) {
      return null;
    }
    return Metric.Level.valueOf(alertStatus);
  }

  public Measure setAlertStatus(Metric.Level level) {
    if (level != null) {
      this.alertStatus = level.toString();
    } else {
      this.alertStatus = null;
    }
    return this;
  }

  public String getData() {
    if (this.textValue != null) {
      return this.textValue;
    }
    if (metric.isDataType() && !measureData.isEmpty()) {
      return measureData.get(0).getText();
    }
    return null;
  }

  public void setData(String data) {
    if (data == null) {
      this.textValue = null;
      measureData.clear();

    } else {
      if (data.length() > TEXT_VALUE_LENGTH) {
        measureData.clear();
        measureData.add(new MeasureData(this, data));

      } else {
        this.textValue = data;
      }
    }
  }

  /**
   * Use getData() instead
   */
  public MeasureData getMeasureData() {
    if (!measureData.isEmpty()) {
      return measureData.get(0);
    }
    return null;
  }

  /**
   * Use setData() instead
   */
  //@Deprecated
  public void setMeasureData(MeasureData data) {
    measureData.clear();
    if (data != null) {
      this.measureData.add(data);
    }
  }

  public String getAlertText() {
    return alertText;
  }

  public void setAlertText(String alertText) {
    this.alertText = alertText;
  }

  @Override
  public String toString() {
    return new ToStringBuilder(this).
        append("value", value).
        append("metric", metric).
        toString();
  }

  public Integer getRuleId() {
    if (getRule() != null) {
      return getRule().getId();
    }
    return null;
  }

  public Double getDiffValue1() {
    return diffValue1;
  }

  public void setDiffValue1(Double diffValue1) {
    this.diffValue1 = diffValue1;
  }

  public Double getDiffValue2() {
    return diffValue2;
  }

  public void setDiffValue2(Double diffValue2) {
    this.diffValue2 = diffValue2;
  }

  public Double getDiffValue3() {
    return diffValue3;
  }

  public void setDiffValue3(Double diffValue3) {
    this.diffValue3 = diffValue3;
  }

  public Measure save(DatabaseSession session) {
    this.metric = session.reattach(Metric.class, metric.getId());
    MeasureData data = getMeasureData();
    setMeasureData(null);
    session.save(this);

    if (data != null) {
      data.setMeasure(session.getEntity(Measure.class, getId()));
      data.setSnapshotId(snapshotId);
      session.save(data);
      setMeasureData(data);
    }
    return this;
  }
}
